@@ -25,37 +25,22 @@
 
 #define __COMPLEX_H
 
+#include <complex.h>
 #include "config.h"
 
-extern COMPLEX_FLOAT quantum_conj(COMPLEX_FLOAT a);
+#define quantum_conj(z) (conj(z))
+#define quantum_real(z) (creal(z))
+#define quantum_imag(z) (cimag(z))
 
-extern float quantum_prob (COMPLEX_FLOAT a);
-extern COMPLEX_FLOAT quantum_cexp(float phi);
-
-/* Return the real part of a complex number */
-
-static inline float
-quantum_real(COMPLEX_FLOAT a)
-{
-  float *p = (float *) &a;
-  return p[0];
-}
-
-/* Return the imaginary part of a complex number */
-
-static inline float
-quantum_imag(COMPLEX_FLOAT a)
-{
-  float *p = (float *) &a;
-  return p[1];
-}
+extern double quantum_prob (COMPLEX_FLOAT a);
+extern COMPLEX_FLOAT quantum_cexp(REAL_FLOAT phi);
 
 /* Calculate the square of a complex number (i.e. the probability) */
 
-static inline float 
+static inline double
 quantum_prob_inline(COMPLEX_FLOAT a)
 {
-  float r, i;
+  REAL_FLOAT r, i;
 
   r = quantum_real(a);
   i = quantum_imag(a);
