@@ -32,9 +32,12 @@ enum {
   QUANTUM_EMLARGE      = 3,
   QUANTUM_EMSIZE       = 4,
   QUANTUM_EHASHFULL    = 5,
+  QUANTUM_EHERMITIAN   = 6, 
+  QUANTUM_ENOCONVERGE  = 7,
+  QUANTUM_ENOSOLVER    = 8,
   QUANTUM_ENOLAPACK    = 32768, /* LAPACK errors start at 32768 */
   QUANTUM_ELAPACKARG   = 32769,
-  QUANTUM_ELAPACKCHEEV = 32770,
+  QUANTUM_ELAPACKCONV  = 32770,
   QUANTUM_EMCMATRIX    = 65536, /* internal errors start at 65536 */
   QUANTUM_EOPCODE      = 65537
 };
